"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const CryptoJs = require("crypto-js");
const Https = require("https");
const SignalR = require("signalr-client");
const url_1 = require("url");
const Market_1 = require("./model/Market");
const Currency_1 = require("./model/Currency");
const Ticker_1 = require("./model/Ticker");
const MarketSummary_1 = require("./model/MarketSummary");
const Order_1 = require("./model/Order");
const OrderBookType_1 = require("./enum/OrderBookType");
const Trade_1 = require("./model/Trade");
const OrderType_1 = require("./enum/OrderType");
const OpenOrder_1 = require("./model/OpenOrder");
const Balance_1 = require("./model/Balance");
const ExchangeStateUpdate_1 = require("./model/ExchangeStateUpdate");
const util_1 = require("util");
const ResponseParsingError_1 = require("./error/ResponseParsingError");
class BittrexApiClient {
    constructor(apiKey, apiSecret) {
        if (util_1.isNullOrUndefined(apiKey) || util_1.isNullOrUndefined(apiSecret)) {
            throw new Error("API key and API secret cannot be undefined or null");
        }
        this.apiKey = apiKey;
        this.apiSecret = apiSecret;
    }
    getMarkets() {
        return __awaiter(this, void 0, void 0, function* () {
            let marketsJson = yield this.makeRequest("public/getmarkets");
            let markets = [];
            for (let marketJson of marketsJson) {
                markets.push(new Market_1.Market(marketJson));
            }
            return markets;
        });
    }
    getCurrencies() {
        return __awaiter(this, void 0, void 0, function* () {
            let currenciesJson = yield this.makeRequest("public/getcurrencies");
            let currencies = [];
            for (let currencyJson of currenciesJson) {
                currencies.push(new Currency_1.Currency(currencyJson));
            }
            return currencies;
        });
    }
    getTicker(market) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Ticker_1.Ticker(yield this.makeRequest("public/getticker", ["market", market]));
        });
    }
    getMarketSummaries() {
        return __awaiter(this, void 0, void 0, function* () {
            let marketSummariesJson = yield this.makeRequest("public/getmarketsummaries");
            let marketSummaries = [];
            for (let marketSummaryJson of marketSummariesJson) {
                marketSummaries.push(new MarketSummary_1.MarketSummary(marketSummaryJson));
            }
            return marketSummaries;
        });
    }
    getMarketSummary(market) {
        return __awaiter(this, void 0, void 0, function* () {
            return new MarketSummary_1.MarketSummary(yield this.makeRequest("public/getmarketsummary", ["market", market])[0]);
        });
    }
    getOrderBook(market, type) {
        return __awaiter(this, void 0, void 0, function* () {
            let ordersJson = yield this.makeRequest("public/getorderbook", ["market", market], ["type", OrderBookType_1.OrderBookType[type]]);
            let orders = [];
            if (type === OrderBookType_1.OrderBookType.BOTH) {
                for (let buyOrderJson of ordersJson.buy) {
                    orders.push(new Order_1.Order(buyOrderJson, OrderType_1.OrderType.BUY));
                }
                for (let sellOrderJson of ordersJson.sell) {
                    orders.push(new Order_1.Order(sellOrderJson, OrderType_1.OrderType.SELL));
                }
                return orders;
            }
            for (let orderJson of ordersJson) {
                ordersJson.push(new Order_1.Order(orderJson, type === OrderBookType_1.OrderBookType.BUY ? OrderType_1.OrderType.BUY : OrderType_1.OrderType.SELL));
            }
            return orders;
        });
    }
    getMarketHistory(market) {
        return __awaiter(this, void 0, void 0, function* () {
            let tradesJson = yield this.makeRequest("/public/getmarkethistory", ["market", market]);
            let trades = [];
            for (let tradeJson of tradesJson) {
                trades.push(new Trade_1.Trade(tradeJson));
            }
            return trades;
        });
    }
    buyWithLimit(market, quantity, rate) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.makeRequest("/market/buylimit", ["market", market], ["quantity", quantity.toString()], ["rate", rate.toString()])).uuid;
        });
    }
    sellWithLimit(market, quantity, rate) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.makeRequest("/market/selllimit", ["market", market], ["quantity", quantity.toString()], ["rate", rate.toString()])).uuid;
        });
    }
    cancelOrder(orderId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.makeRequest("/market/cancelOrder", ["uuid", orderId]);
            return true;
        });
    }
    getOpenOrders(market) {
        return __awaiter(this, void 0, void 0, function* () {
            let openOrdersJson = yield this.makeRequest("/market/getopenorders", ["market", market]);
            let openOrders = [];
            for (let openOrderJson of openOrdersJson) {
                openOrders.push(new OpenOrder_1.OpenOrder(openOrderJson));
            }
            return openOrders;
        });
    }
    getBalances() {
        return __awaiter(this, void 0, void 0, function* () {
            let balancesJson = yield this.makeRequest("/account/getbalances");
            let balances = [];
            for (let balanceJson of balancesJson) {
                balances.push(new Balance_1.Balance(balanceJson));
            }
            return balances;
        });
    }
    getBalance(currency) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Balance_1.Balance(yield this.makeRequest("/account/getbalance", ["currency", currency]));
        });
    }
    getDepositAddress(currency) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.makeRequest("/account/getdepositaddress", ["currency", currency])).Address;
        });
    }
    withdraw(currency, quantity, address, paymentId) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.makeRequest("/account/withdraw", ["currency", currency], ["quantity", quantity.toString()], ["address", address], ["paymentId", paymentId])).uuid;
        });
    }
    getExchangeStateUpdatesStream(watchableMarkets, callback) {
        this.websocketClient = new SignalR.client("wss://socket.bittrex.com/signalr", ["CoreHub"]);
        this.websocketClient.serviceHandlers.connected = () => {
            for (let watchableMarket of watchableMarkets) {
                this.websocketClient.call("CoreHub", "SubscribeToExchangeDeltas", watchableMarket);
            }
        };
        this.websocketClient.serviceHandlers.messageReceived = (messageJson) => {
            if (messageJson.type !== "utf8") {
                return;
            }
            try {
                messageJson = JSON.parse(messageJson.utf8Data);
            }
            catch (error) {
                throw new ResponseParsingError_1.ResponseParsingError(`An error occurred parsing Bittrex's response. The response was: ${messageJson}`);
            }
            let updatesJson = messageJson.M;
            if (updatesJson === undefined || updatesJson.length === 0) {
                return;
            }
            let exchangeStateUpdates = [];
            for (let updateJson of updatesJson) {
                if (updateJson.M !== "updateExchangeState") {
                    continue;
                }
                for (let exchangeStateUpdateJson of updateJson.A) {
                    exchangeStateUpdates.push(new ExchangeStateUpdate_1.ExchangeStateUpdate(exchangeStateUpdateJson));
                }
            }
            if (exchangeStateUpdates.length === 0) {
                return;
            }
            callback(exchangeStateUpdates);
        };
    }
    makeRequest(operation, ...parameters) {
        return __awaiter(this, void 0, void 0, function* () {
            let apiEndpointUrl = new url_1.URL("/api/v1.1/" + operation, "https://www.bittrex.com/");
            apiEndpointUrl.searchParams.append("apikey", this.apiKey);
            apiEndpointUrl.searchParams.append("nonce", new Date().getTime().toString());
            for (let parameter of parameters) {
                if (parameter === undefined || parameter === null) {
                    continue;
                }
                apiEndpointUrl.searchParams.append(parameter[0], parameter[1]);
            }
            let apiSign = CryptoJs.HmacSHA512(apiEndpointUrl.toString(), this.apiSecret);
            return new Promise((fulfill, reject) => {
                let clientRequest = Https.request(apiEndpointUrl, (bittrexResponse) => {
                    bittrexResponse.on("data", (bittrexData) => {
                        try {
                            bittrexData = JSON.parse(bittrexData.toString());
                        }
                        catch (error) {
                            throw new ResponseParsingError_1.ResponseParsingError(`An error occurred parsing Bittrex's response. The response was: ${bittrexData.toString()}`);
                        }
                        if (bittrexData.success) {
                            fulfill(bittrexData.result);
                        }
                        else {
                            reject(bittrexData.message);
                        }
                    });
                });
                clientRequest.setHeader("apisign", apiSign);
                clientRequest.end();
            });
        });
    }
}
exports.BittrexApiClient = BittrexApiClient;
//# sourceMappingURL=BittrexApiClient.js.map